
# camera animation functions
# to be called at each frame from any plugin to make camera animation

require 'sketchup.rb'

module Camera_Animation

	def self.rotate(ratio, altitude, radius, precision)
	
		# sample : Camera_Animation.rotate(0.96, 10, 70, 1.5)
		
		# the camera will rotate around the selection
		# ratio : how fast the camera will move from previous position to the new one. From 0.0 to 1.0. Good values around 0.95.
		# altitude : relative z position of the camera from the target. In inches. 
		# radius : distance from the eye to the target in inches
		# precision : angle in degrees that the camera will rotate at each frame
	
		selection = Sketchup.active_model.selection
		if selection.length == 0
			return
		end
		view = Sketchup.active_model.active_view

		camera1 = view.camera
		eye1 = camera1.eye
		target1 = camera1.target
		up1 = camera1.up
		
		if @angle == nil
			@angle = 0
		end
		
		@angle = @angle + precision
		
		target2 = selection[0].bounds.center
		eye2 = target2.clone
		#eye2.z = target1.z + altitude
		eye2.z = ((target2.z + altitude) * ratio) + (eye2.z * (1.0 - ratio))
		t = Geom::Transformation.translation Geom::Vector3d.new(radius, 0, 0)
		eye2.transform! t
		t = Geom::Transformation.rotation(target2, Geom::Vector3d.new(0,0,1), @angle.degrees)
		eye2.transform! t
		up2 = Geom::Vector3d.new(0,0,1)

		eye = Geom::Point3d.linear_combination ratio, eye1, (1.0 - ratio), eye2
		target = Geom::Point3d.linear_combination ratio, target1, (1.0 - ratio), target2
		up = up2
		
		camera = Sketchup::Camera.new eye, target, up
		view.camera = camera
		
	end

	def self.target(ratio, altitude, focal)
	
		# sample : Camera_Animation.target(0.97, 15, 100)
		
		# the camera will focus target and adjust focal
		# ratio : how fast the camera will move from previous position to the new one. From 0.0 to 1.0. Good values around 0.95.
		# altitude : relative z position of the camera from the target. In inches. 
		# focal : focal to be reached
		
		selection = Sketchup.active_model.selection
		if selection.length == 0
			return
		end
		view = Sketchup.active_model.active_view
		
		camera1 = view.camera
		eye1 = camera1.eye
		target1 = camera1.target
		up1 = camera1.up
		focal1 = camera1.focal_length
		
		target2 = selection[0].bounds.center
		eye2 = eye1.clone
		#eye2.z = target1.z + altitude
		eye2.z = ((target2.z + altitude) * ratio) + (eye2.z * (1.0 - ratio))
		up2 = Geom::Vector3d.new(0,0,1)
		focal2 = focal
		
		eye = Geom::Point3d.linear_combination ratio, eye1, (1.0 - ratio), eye2
		target = Geom::Point3d.linear_combination ratio, target1, (1.0 - ratio), target2
		focal = (focal1 * ratio) + (focal2 * (1.0 - ratio))
		up = up2

		camera = Sketchup::Camera.new eye, target, up
		camera.focal_length = focal
		view.camera = camera
		
	end

	def self.follow(ratio, altitude)
	
		# sample : Camera_Animation.follow(0.9, 10)
		
		# the camera will follow the target
		# ratio : how fast the camera will move from previous position to the new one. From 0.0 to 1.0. Good values around 0.95.
		# altitude : relative z position of the camera from the target. In inches. 
	
		selection = Sketchup.active_model.selection
		if selection.length == 0
			return
		end
		view = Sketchup.active_model.active_view

		camera1 = view.camera
		eye1 = camera1.eye
		target1 = camera1.target
		up1 = camera1.up
		
		target2 = selection[0].bounds.center
		eye2 = Geom::Point3d.linear_combination ratio, eye1, (1.0 - ratio), target2
		eye2.z = ((target2.z + altitude) * ratio) + (eye2.z * (1.0 - ratio))
		up2 = Geom::Vector3d.new(0,0,1)
		
		eye = Geom::Point3d.linear_combination ratio, eye1, (1.0 - ratio), eye2
		target = Geom::Point3d.linear_combination ratio, target1, (1.0 - ratio), target2
		up = up2

		camera = Sketchup::Camera.new eye, target, up
		view.camera = camera
		
	end
	
end # end of Camera_Animation module

